﻿// This sample will demonstrate how to call a server function using LANSA Open for .NET.
// The standard DEMO table PSLEVENT is used. This table will be populated with test data.
// This example will demonstrate how to exchange fields and working lists to and from the 
// server function.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using HR;
using LOpen.EntityModel;

namespace CallAServerFunction
{
	public partial class MainForm : Form
	{
		private readonly DataContext masterContext;
		private Department[] departments;

		public MainForm(Util.ConnectionForm cf)
		{
			InitializeComponent();

			// Create a master context (non-updatable)
			// Used to retrieve readonly data such as department list/employee list
			masterContext = new DataContext(false);
			masterContext.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password);

			// Populate the events table with some data for the purposes of this example.
			PopulateEventsTable();

			// Populate the department list from the DEPTAB table.
			// Query the database table
			departments = masterContext.Departments.RetrieveList();

			// Populate the combobox
			int index;
			foreach (Department department in departments)
			{
				index = cmbDepartment.Items.Add(department.Description);
			}

			cmbDepartment.SelectedIndex = 0;
		}

		private void pubGetEvents_Click(object sender, EventArgs e)
		{
			lvwEmployeeTotals.Items.Clear();
			lvwEvents.Items.Clear();

			var info = masterContext.GetEmployeeEvents.CreateInvokeInfo();

			// Add a dummy record to show that the list contents are passed to the server
			var empEventItem = info.Lists.EmployeeEvents.NewItem();
			empEventItem.EmployeeNumber = "Dummy";
			empEventItem.EmployeeEventCount = 99999;
			info.Lists.EmployeeEvents.Add(empEventItem);

			// Call the server function passing the department whose events are to be returned.
			info.In.Department = departments[cmbDepartment.SelectedIndex].Id;
			masterContext.GetEmployeeEvents.Invoke(info);

			// Move the values from the Working List to the listView control using an indexed
			// for loop. The field names in the EmployeeEvents working list have been renamed
			// in the LANSA Model to more meaningful names.
			String[] listData = new String[2];
			ListViewItem listRow;
			int count = info.Lists.EmployeeEvents.Count;
			for (int i = 0; i < count; i++)
			{
				listData[0] = info.Lists.EmployeeEvents[i].EmployeeNumber;
				listData[1] = info.Lists.EmployeeEvents[i].EmployeeEventCount.ToString();
				listRow = new ListViewItem(listData);
				lvwEmployeeTotals.Items.Add(listRow);
			}

			// Move the values from the Working List to the listView control using a
			// foreach loop. The field names in the Events list have not been renamed
			// in the LANSA Model to provide a contrast to the EmployeeEvents list.
			listData = new String[4];
			foreach (var eventItem in info.Lists.Events)
			{
				listData[0] = eventItem.EMPNO;
				listData[1] = eventItem.EMPEVTNUM.ToString();
				listData[2] = eventItem.EMPEVTDAT.ToShortDateString();
				listData[3] = eventItem.EMPEVTNTE;
				listRow = new ListViewItem(listData);
				lvwEvents.Items.Add(listRow);
			}

			lblTotalEntries.Text = String.Concat("Total Entries: ", info.Out.TotalEvents.ToString());
			lblTimestamp.Text = String.Concat("Queried at ", info.Out.TimeStamp.ToString());
		}

		private void PopulateEventsTable()
		{
			String[] eventTypes = { "AWD", "NOT", "PRO", "WRN" };

			var eventContext = new DataContext(masterContext, true);
			Event evnt;

			// Retrieve the list of employees as the employee associated with the event
			// must be a valid employee.
			var columns = EmployeeCols.EmployeeNo + EmployeeCols.FirstName + EmployeeCols.Surname;
			var employees = masterContext.Employees.RetrieveList(columns);

			short totalEvents = 1;
			foreach (var employee in employees)
			{
				for (short i = 0; i < totalEvents; i++)
				{
					// Can't use "event" as a variable name as its a C# keyword
					evnt = new Event();

					evnt.EMPNO = employee.EmployeeNo;
					evnt.EMPEVTNUM = i + 999990;
					evnt.EMPEVTDAT = DateTime.Now;
					evnt.EMPEVTTIM = DateTime.Now;
					evnt.EMPEVTTYP = eventTypes[i];
					evnt.EMPEVTNTE = "This record has been inserted by the LANSA Open.Net sample application";
					evnt.EMPEVTUPD = i * 50;
					evnt.EMPEVTUDT = DateTime.Now;
					evnt.EMPEVTDOC = null;

					eventContext.Attach(evnt);
				}

				if (totalEvents < 4) { totalEvents++; } else { totalEvents = 1; }
			}

			try
			{
				eventContext.SubmitChanges();
			}
			catch (ApplyChangesException ex)
			{
				// Handle conflicts but ignore other errors for this test.
				foreach (var changeConflict in ex.ChangeConflicts)
				{
					changeConflict.Resolve(ConflictResolveMode.OverwriteCurrentValues);
				}
			}
			catch (System.Exception ex)
			{
				MessageBox.Show(ex.Message, "Error populating PSLEVENT");
			}
		}
	}
}
