/**
 * This sample demonstrates how to resolve update-conflicts.
 * 
 * An update conflict occurs when a record that a user is trying to update is no longer up-to-date
 * as it has been changed by another user.
 * When a conflict happens, it has to be resolved before our changes can be applied to the database.
 * 
 * A conflict is resolved by indicating which values (our in-memory values vs. database values) should be
 * kept and which ones should be discarded.
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using HR;
using LOpen.EntityModel;

namespace ChangeConflictResolution
{
    public partial class MainForm : Form
    {
        private DataContext context1 = null;
        private DataContext context2 = null;

        private Employee emp1 = null;
        private Employee emp2 = null;

        public MainForm(Util.ConnectionForm cf)
        {
			// Create 2 contexts to simulate 2 different users

			// Create context 1 (= user 1)
            context1 = new DataContext(true);
            context1.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password);

			// Set UpdateModifiedColumnsOnly to false to ensure that all columns will be checked
			// for update-conflicts when attempting to change their values.
			// If UpdateModifiedColumnsOnly is false, only columns whose in-memory (local) values have been changed
			// will be checked for update-conflict.
			context1.Options.UpdateModifiedColumnsOnly = false;

			// Create context 2 (= user 2)
            context2 = new DataContext(true);
			context2.Options.UpdateModifiedColumnsOnly = false;
			context2.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password);

            //
            InitializeComponent();
            PopulateInstances(0);
        }

		// Retrieve employee details from the server
        private void PopulateInstances(int instance)
        {
            const string empNo = "A1015";

            // User 1
            if (instance == 0 || instance == 1)
            {
                emp1 = context1.Employees.RetrieveItem(empNo);
				if (emp1 != null)
				{
					boxEmpNo1.Text = emp1.EmployeeNo;
					boxFirstName1.Text = emp1.FirstName;
					boxSurname1.Text = emp1.Surname;
					boxSalary1.Value = emp1.Salary;
				}
            }

            // User 2
            if (instance == 0 || instance == 2)
            {
                emp2 = context2.Employees.RetrieveItem(empNo);
				if (emp2 != null)
				{
					boxEmpNo2.Text = emp2.EmployeeNo;
					boxFirstName2.Text = emp2.FirstName;
					boxSurname2.Text = emp2.Surname;
					boxSalary2.Value = emp2.Salary;
				}
            }

			//
			if (emp1 == null || emp2 == null) MessageBox.Show("Employee " + empNo + " does not exist", "Error");
        }

		// Save employee details of User 1
        private void btnSave1_Click(object sender, EventArgs e)
        {
			if (emp1 != null)
			{
				emp1.FirstName = boxFirstName1.Text;
				emp1.Surname = boxSurname1.Text;
				emp1.Salary = boxSalary1.Value;

				SubmitChanges(context1);
				PopulateInstances(1);
			}
        }

		// Save employee details of User 2
        private void btnSave2_Click(object sender, EventArgs e)
        {
			if (emp2 != null)
			{
				emp2.FirstName = boxFirstName2.Text;
				emp2.Surname = boxSurname2.Text;
				emp2.Salary = boxSalary2.Value;

				SubmitChanges(context2);
				PopulateInstances(2);
			}
        }

		// Apply changes to the server
        private void SubmitChanges(DataContext context)
        {
            bool repeat;
            do
            {
                repeat = false;

                //
                try
                {
                    context.SubmitChanges();
                    MessageBox.Show("Update successful!");
                }
                catch (ApplyChangesException ex)
                {
					// Check if update-conflicts have occurred
                    if (ex.ChangeConflicts.Count > 0)
                    {
						// Display the conflict-resolution form
                        using (ConflictForm form = new ConflictForm(ex))
                        {
                            DialogResult result = form.ShowDialog();
                            if (result == DialogResult.OK) repeat = true;
                        }
                    }
                    else
                    {
                        MessageBox.Show(ex.Message);
                    }
                }
            } while (repeat);
        }
    }
}