﻿/**
 * This sample application demonstrates how to cache data retrieved from the server
 * using context serialization.
 * 
 * In this sample, records retrieved from the server will be serialized and then
 * written to the cache which is simply just a file.
 **/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using LOpen.EntityModel;
using Model;
using Util;

namespace DataCaching
{
	public partial class MainForm : Form
	{
		private readonly DataContext context;

		public MainForm()
		{
			InitializeComponent();

			//
			context = new DataContext(true);
			RefreshUI();
		}

		// Enable/disable buttons to reflect the current state of the application (online/offline)
		private void RefreshUI()
		{
			bool connected = context.IsConnected;

			this.btnConnect.Enabled = !connected;
			this.btnDisconnect.Enabled = connected;
			this.btnSaveToCache.Enabled = connected;

			this.lblState.Text = connected ? "Connected" : "Offline";
			this.btnDisplayRecords.Text = connected ? "Display Live Data" : "Display Cached Data";
		}

		// Go online
		private void btnConnect_Click(object sender, EventArgs e)
		{
			using (ConnectionForm cf = new ConnectionForm())
			{
				// Show connection settings dialog box 
				if (cf.ShowDialog() == DialogResult.OK)
				{
					try
					{
						// Connect to the server
						context.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password);

						// Enable/disable buttons appropriately
						RefreshUI();
					}
					catch (Exception ex)
					{
						MessageBox.Show(ex.Message, "Error");
					}
				}
			}
		}

		// Go offline
		private void btnDisconnect_Click(object sender, EventArgs e)
		{
			// Disconnect from the server
			context.Disconnect();
			context.Clear();

			// Enable/disable buttons appropriately
			RefreshUI();
		}

		// Write records to cache
		private void btnSaveToCache_Click(object sender, EventArgs e)
		{
			context.Clear();

			// Retrieve records off the server
			var depts = context.Departments.RetrieveList();
			var sects = context.Sections.RetrieveList();

			// Add retrieved records to the context's dataset
			context.DataSet.Add(depts);
			context.DataSet.Add(sects);

			// Serialize the context
			string serialized = context.Serialize();

			// Write the serialized context to a text file called 'cache.txt'
			string appDir = Path.GetDirectoryName(Application.ExecutablePath);
			string cachePath = Path.Combine(appDir, "cache.txt");
			File.WriteAllText(cachePath, serialized);

			//
			MessageBox.Show("The cache file has been created successfully.", "Cache");
		}

		// Display Department & Section records on the datagrids
		// If currently online, records will be retrieved from the server
		// If currently offline, records stored in the cache file will be used
		private void btnDisplayRecords_Click(object sender, EventArgs e)
		{
			bool connected = context.IsConnected;

			//
			try
			{
				// If online, set datasource to the server's database
				if (connected)
				{
					context.Options.DataSource = DataSourceType.Database;
				}
				else
				{
					// If offline, read the cache file ...
					string appDir = Path.GetDirectoryName(Application.ExecutablePath);
					string cachePath = Path.Combine(appDir, "cache.txt");

					if (File.Exists(cachePath))
					{
						string serialized = File.ReadAllText(cachePath);

						// ... and reconstruct (deserialize) the context
						context.Deserialize(serialized);

						// Set datasource to the context's dataset
						context.Options.DataSource = DataSourceType.Dataset;
					}
					else
					{
						throw new Exception("The cache has not yet been created.");
					}
				}

				// Depending on the datasource, RetrieveList will either retrieve records
				// from the server or the context's dataset.
				dataGrid1.DataSource = context.Departments.RetrieveList();
				dataGrid2.DataSource = context.Sections.RetrieveList();
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message, "Error");
			}
		}

		// Clear the datagrids
		private void btnClearData_Click(object sender, EventArgs e)
		{
			context.Clear();

			dataGrid1.DataSource = null;
			dataGrid2.DataSource = null;
		}
	}
}
