﻿/**
 * This sample illustrates how to reference a table and perform operations 
 * in a generic manner.
 **/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using LOpen.EntityModel;
using Model;

namespace GenericTableOperations
{
	public partial class MainForm : Form
	{
		private readonly DataContext context;

		public MainForm(Util.ConnectionForm cf)
		{
			InitializeComponent();

			// Connect to server
			context = new DataContext(false);
			context.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password);

			// Populate tables drop down
			PopulateTables();
		}

		/// <summary>
		/// Get all tables defined in our data model and 
		/// populate the drop down on the form with those tables.
		/// </summary>
		private void PopulateTables()
		{
			// Use 'GetAllTables' function of the context
			// GetAllTables returns an array of 'ITable'
			foreach (var table in context.GetAllTables())
			{
				// Add to the drop down
				this.cbTables.Items.Add(new ListItem(table, table.Name));
			}
		}

		/// <summary>
		/// Get the currently selected table on the drop down.
		/// </summary>
		private ITable GetSelectedTable()
		{
			ITable result = null;

			//
			if (this.cbTables.SelectedItem != null)
			{
				var listItem = this.cbTables.SelectedItem as ListItem;
				result = listItem.AssociatedObject as ITable;
			}

			//
			return result;
		}		

		/// <summary>
		/// Different table on the drop down is selected
		/// </summary>
		private void cbTables_SelectedIndexChanged(object sender, EventArgs e)
		{
			// Get selected table
			var selectedTable = GetSelectedTable();

			// Populate the column list for this table
			this.lstColumns.Items.Clear();
			if (selectedTable != null)
			{
				// Use the 'GetAllColumns' method of the 'ITable' interface
				// GetAllColumns returns an array of 'Column'
				var columns = selectedTable.GetAllColumns();

				// Iterate through the list of columns & 
				// populate the list on the form.
				foreach (var column in columns)
				{
					string text = column.Description + " (" + column.Name + ")";
					this.lstColumns.Items.Add(new ListItem(column, text), true);
				}
			}
		}

		/// <summary>
		/// Retrieve records off the server
		/// </summary>
		private void btnRetrieveList_Click(object sender, EventArgs e)
		{
			// Get the selected table
			var selectedTable = GetSelectedTable();

			//
			if (selectedTable == null)
			{
				MessageBox.Show("There is no table selected", "Error");
			}
			else
			{
				var columns = new List<Column>();

				//
				dataGrid.Columns.Clear();
				dataGrid.AutoGenerateColumns = false;

				// Include checked columns in the data grid
				foreach (ListItem listItem in this.lstColumns.CheckedItems)
				{
					var column = listItem.AssociatedObject as Column;
					columns.Add(column);

					//
					var dgColumn = new DataGridViewColumn();
					dgColumn.CellTemplate = new DataGridViewTextBoxCell();
					dgColumn.HeaderText = column.Description;
					dgColumn.DataPropertyName = column.PropertyName;
					dataGrid.Columns.Add(dgColumn);
				}

				// Get the WHERE clause
				string cond = this.txtCondition.Text.Trim().ToUpperInvariant();

				//
				try
				{
					// Retrieve records from the server using the 'RetrieveList' method of 'ITable',
					// passing the expected columns and WHERE clause.
					var list = selectedTable.RetrieveList(columns.ToArray(), cond);

					// Use the resulting list as the data source for the grid on the form.
					dataGrid.DataSource = list;
				}
				catch (Exception ex)
				{
					MessageBox.Show(ex.Message, "Error");
				}
			}
		}

		/// <summary>
		/// Represents an item in a list (dropdown or list-box) on the form.
		/// </summary>
		class ListItem
		{
			public readonly object AssociatedObject;
			private readonly string text;

			public ListItem(object associatedObject, string text)
			{
				this.AssociatedObject = associatedObject;
				this.text = text;
			}

			public override string ToString()
			{
				return this.text;
			}
		}
	}
}
