using System;
using System.Data;
using System.Configuration;
using System.Collections.Generic;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Specialized;

using LOpen.EntityModel;
using HR;
using System.Text;


namespace HRWebApplication
{
	/// <summary>
	/// This is the main application's web page.
	/// </summary>
    public partial class _MainPage : System.Web.UI.Page
    {
        private DataContext masterContext = null;
        private DataContext updateContext = null;

        private Employee[] allEmployees = null;
        private Employee currentEmployee = null;

		/// <summary>
		/// Connect to the server
		/// </summary>
        private void Connect()
        {
            var context = masterContext;

            var prms = Request.QueryString;

			// Configure the settings based on the parameters passed in the query string
            context.Settings.Server = prms["server"];
            context.Settings.PortNumber = int.Parse(prms["port"]);
            context.Settings.Partition = prms["partition"];

			// Open connection
            context.Connect(prms["user"], prms["pwd"]);
        }

		/// <summary>
		/// Disconnect from the server
		/// </summary>
        private void Disconnect()
        {
            masterContext.Disconnect();
        }

		/// <summary>
		/// Handles page-load event.
		/// </summary>
        protected void Page_Load(object sender, EventArgs e)
        {
			// Do not cache web pages
            Response.Cache.SetCacheability(HttpCacheability.NoCache);

            // Create a new non-updatable context
            masterContext = new DataContext(false);

            // Check if we have the list of all employees in the view-state
			// If not, retrieve from the server ...
			// ... otherwise use list of employees stored in the view-state
            if (ViewState["employees"] == null)
            {
                // Retrieve employees from server
                RetrieveEmployeeList();
                UpdateDataEntryPanelState();
            }
            else
            {
                // Recreate employee list from serialized context stored in the view-state
                string serializedString = (string)ViewState["employees"];
                allEmployees = masterContext.DeserializeObjects<Employee>(serializedString);
            }

            // Get update context & current employee from the view-state
            string updateContextStr = (string)ViewState["updateContext"];
            if (updateContextStr != null)
            {
                updateContext = DataContext.CreateFromSerialized(masterContext, updateContextStr);

				// The current employee is stored as the first object in the ObjectReferenceStore.
				// See the PopulateDataEntryPanel function to understand how the employee ...
				// ... gets to be stored in the ObjectReferenceStore.
                currentEmployee = updateContext.ObjectReferenceStore[0] as Employee;
            }

            //
            lblMessage.Visible = false;
        }

		/// <summary>
		/// Retrieve the employee list from the server and store them in the view-state for subsequent use.
		/// </summary>
        private void RetrieveEmployeeList()
        {
            // Retrieve list of employees from the server
            Connect();
            try
            {
                // Retrieve the list of employees
                allEmployees = masterContext.Employees.RetrieveList(EmployeeCols.EmployeeNo + EmployeeCols.FirstName + EmployeeCols.Surname);
                
                // Store the employees in the view-state so subsequent requests
                // don't have to retrieve them again from the server.
				// Use the 'Serialize' method of the masterContext to serialize it into a string ...
				// ... and then store the string in the view-state.
                ViewState["employees"] = masterContext.Serialize(allEmployees);

                // Bind the list to the grid-view
                this.employeeList.DataSource = allEmployees;
                this.employeeList.DataBind();
            }
            finally
            {
                Disconnect();
            }
        }

		/// <summary>
		/// Enable or disable the input fields & buttons based on if the data entry screen
		/// is in edit mode or not.
		/// </summary>
        private void UpdateDataEntryPanelState()
        {
            bool hasCurrentEmployee = (this.employeeList.SelectedIndex >= 0);
            this.dataEntryPanel.Visible = hasCurrentEmployee;

            //
            bool editMode = ViewState["EditMode"] != null;
            this.editButton.Visible = !editMode;
            this.saveButton.Visible = editMode;
            this.cancelButton.Visible = editMode;

            //
            this.boxFirstName.ReadOnly = !editMode;
            this.boxSurname.ReadOnly = !editMode;

            //
            if (hasCurrentEmployee)
            {
                if (this.skillList.AutoGenerateEditButton != editMode)
                {
                    this.skillList.AutoGenerateEditButton = editMode;
                    this.skillList.AutoGenerateDeleteButton = editMode;
                    this.skillList.DataSource = currentEmployee.Skills;
                    this.skillList.DataBind();
                }
            }
            else
            {
                updateContext = null;
                currentEmployee = null;
            }
        }

		/// <summary>
		/// Retrieve an employee from the server and put the details in the data entry screen.
		/// </summary>
		/// <param name="employeeNo"></param>
        private void PopulateDataEntryPanel(string employeeNo)
        {
            updateContext = new DataContext(masterContext, true);

            //
            Connect();
            try
            {
                // Retrieve the selected employee from the server
                var employee = updateContext.Employees.RetrieveItem(employeeNo);
                updateContext.ObjectReferenceStore.Add(employee);

                // Populate the UI controls
                this.boxFirstName.Text = employee.FirstName;
                this.boxSurname.Text = employee.Surname;
                this.skillList.DataSource = employee.Skills;
                this.skillList.DataBind();

                //
                currentEmployee = employee;
            }
            finally
            {
                Disconnect();
            }
        }

		/// <summary>
		/// Handle the delete skill event.
		/// </summary>
		protected void skillList_RowDeleting(object sender, GridViewDeleteEventArgs e)
		{
			if (currentEmployee != null)
			{
				// Get the skill the user is deleting
				var skill = currentEmployee.Skills[e.RowIndex];

				// Remove the skill from the current employee
				currentEmployee.Skills.Remove(skill);

				//
				this.skillList.DataSource = currentEmployee.Skills;
				this.skillList.DataBind();
			}
		}

		/// <summary>
		/// Make sure we store the updateContext in the view-state.
		/// We have to maintain the updateContext between requests so that
		/// the context can track the changes made to the data objects in the memory)
		/// (otherwise SubmitChanges will not know if any changes have been made to the objects or not).
		/// </summary>
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);

			//
			if (updateContext != null)
			{
				ViewState["updateContext"] = updateContext.Serialize();
			}
			else
			{
				ViewState["updateContext"] = null;
			}
		}

        protected void editButton_Click(object sender, EventArgs e)
        {
            ViewState["EditMode"] = true;
            UpdateDataEntryPanelState();
        }

        protected void saveButton_Click(object sender, EventArgs e)
        {
            if (currentEmployee != null)
            {
                // Update the employee details
                currentEmployee.FirstName = this.boxFirstName.Text;
                currentEmployee.Surname = this.boxSurname.Text;

                //
                bool updateSuccessful = false;

                //
                Connect();
                try
                {
                    // Submit changes to server
                    updateContext.SubmitChanges();

                    //
                    updateSuccessful = true;
                }
                catch (ApplyChangesException ex)
                {
                    StringBuilder sb = new StringBuilder();

                    // change-conflict
                    if (ex.ChangeConflicts.Count > 0)
                    {
                        sb.AppendLine("The record has been changed by somebody else.");
                    }

                    // validation errors
                    foreach (var info in ex.Errors)
                    {
                        sb.AppendLine(info.Message);
                    }

                    //
                    lblMessage.Text = sb.ToString();
                    lblMessage.Visible = true;
                }
                finally
                {
                    Disconnect();
                }

                //
                if (updateSuccessful)
                {
                    // Remove edit mode indicator
                    ViewState.Remove("EditMode");
                    UpdateDataEntryPanelState();

                    // Update employee list
                    int pageIndex = employeeList.PageIndex;
                    RetrieveEmployeeList();
                    employeeList.PageIndex = pageIndex;
                }
            }
        }

        protected void skillList_RowEditing(object sender, GridViewEditEventArgs e)
        {
            this.skillList.EditIndex = e.NewEditIndex;
            this.skillList.DataSource = currentEmployee.Skills;
            this.skillList.DataBind();
        }

        protected void skillList_RowUpdating(object sender, GridViewUpdateEventArgs e)
        {
            this.skillList.EditIndex = -1;

            //
            TextBox skillCodeBox = (TextBox)this.skillList.Rows[e.RowIndex].Cells[1].Controls[0];
            TextBox gradeBox = (TextBox)this.skillList.Rows[e.RowIndex].Cells[2].Controls[0];

            var skill = currentEmployee.Skills[e.RowIndex];
            skill.SkillCode = skillCodeBox.Text;
            skill.Grade = gradeBox.Text;

            //
            this.skillList.DataSource = currentEmployee.Skills;
            this.skillList.DataBind();
        }

        protected void skillList_RowCancelingEdit(object sender, GridViewCancelEditEventArgs e)
        {
            this.skillList.EditIndex = -1;
            this.skillList.DataSource = currentEmployee.Skills;
            this.skillList.DataBind();
        }

        protected void cancelButton_Click(object sender, EventArgs e)
        {
            if (currentEmployee != null)
            {
				// Reload the current employee from the server
                PopulateDataEntryPanel(currentEmployee.EmployeeNo);

                // Remove edit mode indicator
                ViewState.Remove("EditMode");
                UpdateDataEntryPanelState();
            }
        }

		protected void employeeList_PageIndexChanging(object sender, GridViewPageEventArgs e)
		{
			// Modify the grid-view page index
			this.employeeList.PageIndex = e.NewPageIndex;
			this.employeeList.SelectedIndex = -1;

			// Set the data source again
			this.employeeList.DataSource = allEmployees;
			this.employeeList.DataBind();

			// Update data entry panel buttons & control states
			UpdateDataEntryPanelState();
		}

		protected void employeeList_SelectedIndexChanged(object sender, EventArgs e)
		{
			string empNo = this.employeeList.SelectedRow.Cells[1].Text;
			PopulateDataEntryPanel(empNo);

			// Update data entry panel buttons & control states
			UpdateDataEntryPanelState();
		}
    }
}
