/**
 * This sample application demonstrates how to write a basic desktop application.
 **/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using HR;
using LOpen.EntityModel;

namespace SimpleDesktopApplication
{
    public partial class MainForm : Form
    {
        private readonly DataContext masterContext;
        private DataContext editContext = null;
        private bool editMode = false;

        public MainForm(Util.ConnectionForm cf)
        {
            InitializeComponent();
            this.employeeGridView.AutoGenerateColumns = false;
            this.employeeGridView.DataSource = new object[0];

            // Associate the controls with columns
            boxFirstName.Tag = EmployeeCols.FirstName;
            boxLastName.Tag = EmployeeCols.Surname;
            boxDepartment.Tag = EmployeeCols.Department;
            boxSection.Tag = EmployeeCols.Section;
            boxSalary.Tag = EmployeeCols.Salary;

            // Create a master context (non-updatable)
            // Used to retrieve readonly data such as department list/employee list
            masterContext = new DataContext(false);
            masterContext.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password);

            // Create an edit context (updatable)
            // This context is used to retrieve and update employee details
            // (data entry screen)
            editContext = new DataContext(masterContext, true);

            // Populate various dropdowns & update data entry screen
            PopulateDropdowns();
            UpdateDataEntryPanelState();
        }

        private void btnSearch_Click(object sender, EventArgs e)
        {
            string surname = boxSurname.Text.ToUpper();

            // Do a partial key search (surname) on ByName_Employees logical table
			var columns = EmployeeCols.EmployeeNo + EmployeeCols.FirstName + EmployeeCols.Surname + EmployeeCols.Department + EmployeeCols.Section;
            var result = masterContext.ByName_Employees.RetrieveList(columns, false, surname);

            // Display the result in the grid view
            this.employeeGridView.DataSource = result;
        }

        // Invoked when an employee is selected in the grid view
        // Retrieve selected employee details and display on the data entry screen
        private void employeeGridView_SelectionChanged(object sender, EventArgs e)
        {
            this.employeeBindingSource.Clear();

            //
            bool hasCurrent = this.employeeGridView.SelectedRows.Count > 0;
            if (hasCurrent)
            {
                // Get the selected employee 'EmployeeNo' value
                var row = this.employeeGridView.SelectedRows[0];
                var employee = (Employee)row.DataBoundItem;

                // Retrieve details of the selected employee
                LoadEmployeeDetails(employee.EmployeeNo);
            }

            // Update data entry screen state
            UpdateDataEntryPanelState();
        }

        // Populate Departments & Sections dropdown
        private Section[] allSections = null;
        private void PopulateDropdowns()
        {
            // Populate 'Departments' dropdown
            boxDepartment.DataSource = masterContext.Departments.RetrieveList();

            // Populate 'Sections' dropdown
            allSections = masterContext.Sections.RetrieveList();
        }

        // Invoked when the user selected a department in the dropdown
        // When a department is selected, the section dropdown should be filtered
        // to display sections under that department
        private void boxDepartment_SelectedValueChanged(object sender, EventArgs e)
        {
            // Get the selected department code
            string value = (string)boxDepartment.SelectedValue;

            // Get the list of all sections
            var sections = allSections;

            // Filter sections, include only section under the department
            if (sections != null)
            {
                var filtered = new List<Section>();
                foreach (var section in sections)
                {
                    // Compare department value
                    if (section.Department == value) filtered.Add(section);
                }

                // Display filtered section list
                boxSection.DataSource = filtered;
                boxSection.SelectedValue = "";
            }
        }

        // Load the details of an employee
        private void LoadEmployeeDetails(string empNo)
        {
            // If empNo is not specified, reload details of current employee
            if (empNo == null)
            {
                var current = employeeBindingSource[0] as Employee;
                if (current != null) empNo = current.EmployeeNo;
            }

            //
            if (empNo != null)
            {
                employeeBindingSource.Clear();

                // Load employee details into editContext
                // so changes made to the details will be 
                // tracked automatically by the editContext
                editContext.Clear();
                var employee = editContext.Employees.RetrieveItem(empNo);

                // Bind the employee details to the UI
                if (employee != null) this.employeeBindingSource.Add(employee);
            }
        }

        // Update enabled & visible states of various controls
        private void UpdateDataEntryPanelState()
        {
            bool hasCurrent = this.employeeBindingSource.Count > 0;

            // Data entry screen visibility
            // Show only if there is an employee selected on the list
            panelDataEntryContainer.Visible = hasCurrent;

            // Edit/Save/Cancel buttons visibility
            btnEdit.Visible = !editMode;
            btnSave.Visible = editMode;
            btnCancel.Visible = editMode;

            // Enable/disable data entry input controls
            foreach (Control control in panelDataEntry.Controls)
            {
                if (control.Tag is Column)
                {
                    if (control is TextBox) ((TextBox)control).ReadOnly = !editMode;
                    else if (control is ComboBox) ((ComboBox)control).Enabled = editMode;
                    control.BackColor = SystemColors.Window;
                }
            }

            // Disable employee grid view & search button when in edit mode
            employeeGridView.Enabled = !editMode;
            btnSearch.Enabled = !editMode;
        }

        private void btnEdit_Click(object sender, EventArgs e)
        {
            // Enter edit mode
            editMode = true;
            UpdateDataEntryPanelState();
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            // Move focus out of the input control so the bound data object will be 
            // updated with the latest value in the control
            boxSurname.Focus();

            // Attempt to submit changes to the server
            // Change conflict might occur - we need to repeat the attempt
            // after conflict has been resolved.
            bool repeat;
            do
            {
                // Initially, do not repeat
                repeat = false;
                try
                {
                    // Attempt to submit changes
                    editContext.SubmitChanges();

                    // Successful
                    MessageBox.Show("The details have been saved successfully.", "Update");

                    // Turn off edit mode
                    editMode = false;
                    LoadEmployeeDetails(null);
                    UpdateDataEntryPanelState();
                }
                catch (ApplyChangesException ex)
                {
                    // Validation errors?
                    if (ex.Errors.Count > 0)
                    {
                        // Look for the main Employee object
                        foreach (var err in ex.Errors)
                        {
                            // Give visual feedback for invalid inputs
                            // by changing the background colour to red
                            if (err.Object is Employee) IndicateInvalidInputs(err);
                        }

                        //
                        MessageBox.Show(ex.Message, "Update Error");
                    }

                    // Change conflicts                       
					// This sample illustrates a simple technique of conflict resolution.
					// For a more elaborate technique, please refer to the 'ChangeConflictResolution' sample project
					// in the the solution.
                    else if (ex.ChangeConflicts.Count > 0)
                    {
                        string message =
                            "This record has been updated by somebody else, keep your changes?\n" +
                            "Press 'Yes' to keep your changes, 'No' to discard your changes";

                        // Ask user whether they want to keep or discard their changes
                        ConflictResolveMode resolution;
                        bool keepMine = MessageBox.Show(message, "Update Conflict", MessageBoxButtons.YesNo) == DialogResult.Yes;

						// Resolve conflict in the way indicated by the user ('keep my value' or 'keep database value')
                        if (keepMine) resolution = ConflictResolveMode.KeepCurrentValues;
                        else resolution = ConflictResolveMode.OverwriteCurrentValues;

                        // Resolve the conflict
                        foreach (var conflict in ex.ChangeConflicts)
                        {
                            conflict.Resolve(resolution);
                        }

                        // Indicate that another attempt should be made
                        repeat = true;
                    }
                }
            }
            while (repeat);
        }

        // Change background color of invalid input controls to red
        // for visual feedback.
        private void IndicateInvalidInputs(DataObjectErrorInfo info)
        {
            List<Column> invalidColumns = new List<Column>(info.Columns);

            //
            foreach (Control control in panelDataEntry.Controls)
            {
                Column column = control.Tag as Column;
                if (column != null)
                {
                    bool isInvalid = invalidColumns.Contains(column);

					// Change the background colour to RED if not valid
                    if (isInvalid) control.BackColor = Color.Red;
                    else control.BackColor = SystemColors.Window;
                }
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            // Turn off edit mode
            editMode = false;
            UpdateDataEntryPanelState();

            // Reload current employee details
            LoadEmployeeDetails(null);
        }
    }
}