'**
'* This sample application demonstrates how to cache data retrieved from the server
'* using context serialization.
'* 
'* In this sample, records retrieved from the server will be serialized and then
'* written to the cache which is simply just a file.
'**

Public Class MainForm
    Private context As DataContext

    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        context = New DataContext(True)

        RefreshUI()
    End Sub

    ' Enable/disable buttons based on whether the application is currently online/offline.
    Private Sub RefreshUI()
        Dim connected As Boolean = context.IsConnected

        Me.btnConnect.Enabled = Not connected
        Me.btnDisconnect.Enabled = connected
        Me.btnSaveToCache.Enabled = connected

        Me.lblState.Text = If(connected, "Connected", "Offline")
        Me.btnDisplayRecords.Text = If(connected, "Display Live Data", "Display Cached Data")
    End Sub

    ' Go online
    Private Sub btnConnect_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnConnect.Click
        Using cf As New ConnectionForm()
            If cf.ShowDialog() = DialogResult.OK Then
                Try
                    ' Connect to the server
                    context.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password)
                    RefreshUI()
                Catch ex As Exception
                    MessageBox.Show(ex.Message, "Error")
                End Try
            End If
        End Using
    End Sub

    ' Go offline
    Private Sub btnDisconnect_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnDisconnect.Click
        ' Disconnect
        context.Disconnect()

        ' Clear the context of any data objects
        context.Clear()

        '
        RefreshUI()
    End Sub

    ' Construct cache
    Private Sub btnSaveToCache_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnSaveToCache.Click
        context.Clear()

        ' Retrieve records from the server
        Dim depts = context.Departments.RetrieveList()
        Dim sects = context.Sections.RetrieveList()

        ' Add the records to the context's dataset
        context.DataSet.Add(depts)
        context.DataSet.Add(sects)

        ' Serialize the context
        Dim serialized As String = context.Serialize()

        ' Save the serialized context to a text file
        Dim appDir As String = Path.GetDirectoryName(Application.ExecutablePath)
        Dim cachePath As String = Path.Combine(appDir, "cache.txt")
        File.WriteAllText(cachePath, serialized)

        '
        MessageBox.Show("The cache file has been created successfully.", "Cache")
    End Sub

    ' Display records in the data grids
    ' If currently online, records are retrieved from the server
    ' If currently offline, records are read from the cache
    Private Sub btnDisplayRecords_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnDisplayRecords.Click
        Dim connected As Boolean = context.IsConnected

        '
        Try
            If connected Then
                ' If currently online, use the server's database as data source
                context.Options.DataSource = DataSourceType.Database
            Else
                ' If currently offline, read the cache
                Dim appDir As String = Path.GetDirectoryName(Application.ExecutablePath)
                Dim cachePath As String = Path.Combine(appDir, "cache.txt")

                ' If cache file exists ...
                If File.Exists(cachePath) Then
                    Dim serialized As String = File.ReadAllText(cachePath)

                    ' Deserialize back into the context object
                    context.Deserialize(serialized)

                    ' Data source is from the context's dataset
                    context.Options.DataSource = DataSourceType.Dataset
                Else
                    Throw New Exception("The cache has not yet been created.")
                End If
            End If

            'Depending on the datasource, RetrieveList will either retrieve records
            'from the server or the context's dataset.
            dataGrid1.DataSource = context.Departments.RetrieveList()
            dataGrid2.DataSource = context.Sections.RetrieveList()
        Catch ex As Exception
            MessageBox.Show(ex.Message, "Error")
        End Try
    End Sub

    Private Sub btnClearData_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnClearData.Click
        context.Clear()

        dataGrid1.DataSource = Nothing
        dataGrid2.DataSource = Nothing
    End Sub

End Class
