﻿Public Partial Class MainPage
    Inherits System.Web.UI.Page

    Private masterContext As DataContext = Nothing
    Private updateContext As DataContext = Nothing

    Private allEmployees As Employee() = Nothing
    Private currentEmployee As Employee = Nothing

    Private Sub Connect()
        Dim context = masterContext

        Dim prms = Request.QueryString

        context.Settings.Server = prms("server")
        context.Settings.PortNumber = Integer.Parse(prms("port"))
        context.Settings.Partition = prms("partition")

        context.Connect(prms("user"), prms("pwd"))
    End Sub

    Private Sub Disconnect()
        masterContext.Disconnect()
    End Sub

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs)
        Response.Cache.SetCacheability(HttpCacheability.NoCache)

        '
        masterContext = New DataContext(False)

        ' First time?
        If ViewState("employees") Is Nothing Then
            ' Retrieve employees from server
            RetrieveEmployeeList()
            UpdateDataEntryPanelState()
        Else
            ' Recreate employee list from serialized context stored in the view-state
            Dim serializedString As String = DirectCast(ViewState("employees"), String)
            allEmployees = masterContext.DeserializeObjects(Of Employee)(serializedString)
        End If

        ' Get update context & current employee from the view-state
        Dim updateContextStr As String = DirectCast(ViewState("updateContext"), String)
        If updateContextStr IsNot Nothing Then
            updateContext = DataContext.CreateFromSerialized(masterContext, updateContextStr)
            currentEmployee = TryCast(updateContext.ObjectReferenceStore(0), Employee)
        End If

        '
        lblMessage.Visible = False
    End Sub

    Protected Sub employeeList_PageIndexChanging(ByVal sender As Object, ByVal e As GridViewPageEventArgs)
        ' Modify the grid-view page index
        Me.employeeList.PageIndex = e.NewPageIndex
        Me.employeeList.SelectedIndex = -1

        ' Set the data source again
        Me.employeeList.DataSource = allEmployees
        Me.employeeList.DataBind()

        ' Update data entry panel buttons & control states
        UpdateDataEntryPanelState()
    End Sub

    Protected Sub employeeList_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs)
        Dim empNo As String = Me.employeeList.SelectedRow.Cells(1).Text
        PopulateDataEntryPanel(empNo)

        ' Update data entry panel buttons & control states
        UpdateDataEntryPanelState()
    End Sub

    Private Sub RetrieveEmployeeList()
        ' Retrieve list of employees from the server
        Connect()
        Try
            ' Retrieve the list of employees
            allEmployees = masterContext.Employees.RetrieveList(EmployeeCols.EmployeeNo + EmployeeCols.FirstName + EmployeeCols.Surname)

            ' Store the employees in the viewstate so subsequent requests
            ' don't have to retrieve them again from the server.
            ViewState("employees") = masterContext.Serialize(allEmployees)

            ' Bind the list to the grid-view
            Me.employeeList.DataSource = allEmployees
            Me.employeeList.DataBind()
        Finally
            Disconnect()
        End Try
    End Sub

    Private Sub UpdateDataEntryPanelState()
        Dim hasCurrentEmployee As Boolean = (Me.employeeList.SelectedIndex >= 0)
        Me.dataEntryPanel.Visible = hasCurrentEmployee

        '
        Dim editMode As Boolean = ViewState("EditMode") IsNot Nothing
        Me.editButton.Visible = Not editMode
        Me.saveButton.Visible = editMode
        Me.cancelButton.Visible = editMode

        '
        Me.boxFirstName.[ReadOnly] = Not editMode
        Me.boxSurname.[ReadOnly] = Not editMode

        '
        If hasCurrentEmployee Then
            If Me.skillList.AutoGenerateEditButton <> editMode Then
                Me.skillList.AutoGenerateEditButton = editMode
                Me.skillList.AutoGenerateDeleteButton = editMode
                Me.skillList.DataSource = currentEmployee.Skills
                Me.skillList.DataBind()
            End If
        Else
            updateContext = Nothing
            currentEmployee = Nothing
        End If
    End Sub

    Private Sub PopulateDataEntryPanel(ByVal employeeNo As String)
        updateContext = New DataContext(masterContext, True)

        '
        Connect()
        Try
            ' Retrieve the selected employee from the server
            Dim employee = updateContext.Employees.RetrieveItem(employeeNo)
            updateContext.ObjectReferenceStore.Add(employee)

            ' Populate the UI controls
            Me.boxFirstName.Text = employee.FirstName
            Me.boxSurname.Text = employee.Surname

            '
            Me.skillList.DataSource = employee.Skills
            Me.skillList.DataBind()

            '
            currentEmployee = employee
        Finally
            Disconnect()
        End Try
    End Sub

    Protected Sub editButton_Click(ByVal sender As Object, ByVal e As EventArgs)
        ViewState("EditMode") = True
        UpdateDataEntryPanelState()
    End Sub

    Protected Sub saveButton_Click(ByVal sender As Object, ByVal e As EventArgs)
        If currentEmployee IsNot Nothing Then
            ' Update the employee details
            currentEmployee.FirstName = Me.boxFirstName.Text
            currentEmployee.Surname = Me.boxSurname.Text

            '
            Dim updateSuccessful As Boolean = False

            '
            Connect()
            Try
                ' Submit changes to server
                updateContext.SubmitChanges()

                '
                updateSuccessful = True
            Catch ex As ApplyChangesException
                Dim sb As New StringBuilder()

                ' change-conflict
                If ex.ChangeConflicts.Count > 0 Then
                    sb.AppendLine("The record has been changed by somebody else.")
                End If

                ' validation errors
                For Each info In ex.Errors
                    sb.AppendLine(info.Message)
                Next

                '
                lblMessage.Text = sb.ToString()
                lblMessage.Visible = True
            Finally
                Disconnect()
            End Try

            '
            If updateSuccessful Then
                ' Remove edit mode indicator
                ViewState.Remove("EditMode")
                UpdateDataEntryPanelState()

                ' Update employee list
                Dim pageIndex As Integer = employeeList.PageIndex
                RetrieveEmployeeList()
                employeeList.PageIndex = pageIndex
            End If
        End If
    End Sub

    Protected Sub skillList_RowEditing(ByVal sender As Object, ByVal e As GridViewEditEventArgs)
        Me.skillList.EditIndex = e.NewEditIndex
        Me.skillList.DataSource = currentEmployee.Skills
        Me.skillList.DataBind()
    End Sub

    Protected Sub skillList_RowUpdating(ByVal sender As Object, ByVal e As GridViewUpdateEventArgs)
        Me.skillList.EditIndex = -1

        '
        Dim skillCodeBox As TextBox = DirectCast(Me.skillList.Rows(e.RowIndex).Cells(1).Controls(0), TextBox)
        Dim gradeBox As TextBox = DirectCast(Me.skillList.Rows(e.RowIndex).Cells(2).Controls(0), TextBox)

        Dim skill = currentEmployee.Skills(e.RowIndex)
        skill.SkillCode = skillCodeBox.Text
        skill.Grade = gradeBox.Text

        '
        Me.skillList.DataSource = currentEmployee.Skills
        Me.skillList.DataBind()
    End Sub

    Protected Sub skillList_RowCancelingEdit(ByVal sender As Object, ByVal e As GridViewCancelEditEventArgs)
        Me.skillList.EditIndex = -1
        Me.skillList.DataSource = currentEmployee.Skills
        Me.skillList.DataBind()
    End Sub

    Protected Overloads Overrides Sub OnPreRender(ByVal e As EventArgs)
        MyBase.OnPreRender(e)

        '
        If updateContext IsNot Nothing Then
            ViewState("updateContext") = updateContext.Serialize()
        Else
            ViewState("updateContext") = Nothing
        End If
    End Sub

    Protected Sub cancelButton_Click(ByVal sender As Object, ByVal e As EventArgs)
        If currentEmployee IsNot Nothing Then
            PopulateDataEntryPanel(currentEmployee.EmployeeNo)

            ' Remove edit mode indicator
            ViewState.Remove("EditMode")
            UpdateDataEntryPanelState()
        End If
    End Sub

    Protected Sub skillList_RowDeleting(ByVal sender As Object, ByVal e As GridViewDeleteEventArgs)
        If currentEmployee IsNot Nothing Then
            Dim skill = currentEmployee.Skills(e.RowIndex)
            currentEmployee.Skills.Remove(skill)
            Me.skillList.DataSource = currentEmployee.Skills
            Me.skillList.DataBind()
        End If
    End Sub

End Class