﻿'**
'* This sample application demonstrates how to write a basic desktop application.
'**

Public Class MainForm
    Private masterContext As DataContext
    Private editContext As DataContext = Nothing
    Private editMode As Boolean = False

    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.employeeGridView.AutoGenerateColumns = False
        Me.employeeGridView.DataSource = New Object(-1) {}

        ' Associate the controls with columns
        boxFirstName.Tag = EmployeeCols.FirstName
        boxLastName.Tag = EmployeeCols.Surname
        boxDepartment.Tag = EmployeeCols.Department
        boxSection.Tag = EmployeeCols.Section
        boxSalary.Tag = EmployeeCols.Salary

        ' Create a master context (non-updatable)
        ' Used to retrieve readonly data such as department list/employee list
        masterContext = New DataContext(False)
        Using cf As New Util.ConnectionForm
            cf.ShowDialog()
            masterContext.Connect(cf.Server, cf.PortNumber, cf.Partition, cf.UserName, cf.Password)
        End Using


        ' Create an edit context (updatable)
        ' This context is used to retrieve and update employee details
        ' (data entry screen)
        editContext = New DataContext(masterContext, True)

        ' Populate various dropdowns & update data entry screen
        PopulateDropdowns()
        UpdateDataEntryPanelState()
    End Sub

    Private Sub btnSearch_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnSearch.Click
        Dim surname As String = boxSurname.Text.ToUpper()

        ' Do a partial key search (surname) on ByName_Employees logical table
        Dim columns = EmployeeCols.EmployeeNo + EmployeeCols.FirstName + EmployeeCols.Surname + EmployeeCols.Department + EmployeeCols.Section
        Dim result = masterContext.ByName_Employees.RetrieveList(columns, False, surname)

        ' Display the result in the grid view
        Me.employeeGridView.DataSource = result
    End Sub

    ' Invoked when an employee is selected in the grid view
    ' Retrieve selected employee details and display on the data entry screen
    Private Sub employeeGridView_SelectionChanged(ByVal sender As Object, ByVal e As EventArgs) Handles employeeGridView.SelectionChanged
        Me.employeeBindingSource.Clear()

        '
        Dim hasCurrent As Boolean = Me.employeeGridView.SelectedRows.Count > 0
        If hasCurrent Then
            ' Get the selected employee 'EmployeeNo' value
            Dim row = Me.employeeGridView.SelectedRows(0)
            Dim employee = DirectCast(row.DataBoundItem, Employee)

            ' Retrieve details of the selected employee
            LoadEmployeeDetails(employee.EmployeeNo)
        End If

        ' Update data entry screen state
        UpdateDataEntryPanelState()
    End Sub

    ' Populate Departments & Sections dropdown
    Private allSections As Section() = Nothing
    Private Sub PopulateDropdowns()
        ' Populate 'Departments' dropdown
        boxDepartment.DataSource = masterContext.Departments.RetrieveList()

        ' Populate 'Sections' dropdown
        allSections = masterContext.Sections.RetrieveList()
    End Sub

    ' Invoked when the user selected a department in the dropdown
    ' When a department is selected, the section dropdown should be filtered
    ' to display sections under that department
    Private Sub boxDepartment_SelectedValueChanged(ByVal sender As Object, ByVal e As EventArgs) Handles boxDepartment.SelectedValueChanged
        ' Get the selected department code
        Dim value As String = DirectCast(boxDepartment.SelectedValue, String)

        ' Get the list of all sections
        Dim sections = allSections

        ' Filter sections, include only section under the department
        If sections IsNot Nothing Then
            Dim filtered = New List(Of Section)()
            For Each section In sections
                ' Compare department value
                If section.Department = value Then
                    filtered.Add(section)
                End If
            Next

            ' Display filtered section list
            boxSection.DataSource = filtered
            boxSection.SelectedValue = ""
        End If
    End Sub

    ' Load the details of an employee
    Private Sub LoadEmployeeDetails(ByVal empNo As String)
        ' If empNo is not specified, reload details of current employee
        If empNo Is Nothing Then
            Dim current = TryCast(employeeBindingSource(0), Employee)
            If current IsNot Nothing Then
                empNo = current.EmployeeNo
            End If
        End If

        '
        If empNo IsNot Nothing Then
            employeeBindingSource.Clear()

            ' Load employee details into editContext
            ' so changes made to the details will be 
            ' tracked automatically by the editContext
            editContext.Clear()
            Dim employee = editContext.Employees.RetrieveItem(empNo)

            ' Bind the employee details to the UI
            If employee IsNot Nothing Then
                Me.employeeBindingSource.Add(employee)
            End If
        End If
    End Sub

    ' Update enabled & visible states of various controls
    Private Sub UpdateDataEntryPanelState()
        Dim hasCurrent As Boolean = Me.employeeBindingSource.Count > 0

        ' Data entry screen visibility
        ' Show only if there is an employee selected on the list
        panelDataEntryContainer.Visible = hasCurrent

        ' Edit/Save/Cancel buttons visibility
        btnEdit.Visible = Not editMode
        btnSave.Visible = editMode
        btnCancel.Visible = editMode

        ' Enable/disable data entry input controls
        For Each control As Control In panelDataEntry.Controls
            If TypeOf control.Tag Is Column Then
                If TypeOf control Is TextBox Then
                    DirectCast(control, TextBox).[ReadOnly] = Not editMode
                ElseIf TypeOf control Is ComboBox Then
                    DirectCast(control, ComboBox).Enabled = editMode
                End If
                control.BackColor = SystemColors.Window
            End If
        Next

        ' Disable employee grid view & search button when in edit mode
        employeeGridView.Enabled = Not editMode
        btnSearch.Enabled = Not editMode
    End Sub

    Private Sub btnEdit_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnEdit.Click
        ' Enter edit mode
        editMode = True
        UpdateDataEntryPanelState()
    End Sub

    Private Sub btnSave_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnSave.Click
        ' Move focus out of the input control so the bound data object will be 
        ' updated with the latest value in the control
        boxSurname.Focus()

        ' Attempt to submit changes to the server
        ' Change conflict might occur - we need to repeat the attempt
        ' after conflict has been resolved.
        Dim repeat As Boolean
        Do
            ' Initially, do not repeat
            repeat = False
            Try
                ' Attempt to submit changes
                editContext.SubmitChanges()

                ' Successful
                MessageBox.Show("The details have been saved successfully.", "Update")

                ' Turn off edit mode
                editMode = False
                LoadEmployeeDetails(Nothing)
                UpdateDataEntryPanelState()
            Catch ex As ApplyChangesException
                ' Validation errors?
                If ex.Errors.Count > 0 Then
                    ' Look for the main Employee object
                    For Each ErrorItem In ex.Errors
                        ' Give visual feedback for invalid inputs
                        ' by changing the background colour to red
                        If TypeOf ErrorItem.Object Is Employee Then
                            IndicateInvalidInputs(ErrorItem)
                        End If
                    Next

                    '
                    MessageBox.Show(ex.Message, "Update Error")

                    ' Change conflicts                       
                ElseIf ex.ChangeConflicts.Count > 0 Then
                    Dim message As String = "This record has been updated by somebody else, keep your changes?" & vbLf & "Press 'Yes' to keep your changes, 'No' to discard your changes"

                    ' Ask user whether they want to keep or discard their changes
                    Dim resolution As ConflictResolveMode
                    Dim keepMine As Boolean = MessageBox.Show(message, "Update Conflict", MessageBoxButtons.YesNo) = DialogResult.Yes

                    If keepMine Then
                        resolution = ConflictResolveMode.KeepCurrentValues
                    Else
                        resolution = ConflictResolveMode.OverwriteCurrentValues
                    End If

                    ' Resolve the conflict
                    For Each conflict In ex.ChangeConflicts
                        conflict.Resolve(resolution)
                    Next

                    ' Indicate that another attempt should be made
                    repeat = True
                End If
            End Try
        Loop While repeat
    End Sub

    ' Change background color of invalid input controls to red
    ' for visual feedback.
    Private Sub IndicateInvalidInputs(ByVal info As DataObjectErrorInfo)
        Dim invalidColumns As New List(Of Column)(info.Columns)

        '
        For Each control As Control In panelDataEntry.Controls
            Dim column As Column = TryCast(control.Tag, Column)
            If column IsNot Nothing Then
                Dim isInvalid As Boolean = invalidColumns.Contains(column)

                If isInvalid Then
                    control.BackColor = Color.Red
                Else
                    control.BackColor = SystemColors.Window
                End If
            End If
        Next
    End Sub

    Private Sub btnCancel_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnCancel.Click
        ' Turn off edit mode
        editMode = False
        UpdateDataEntryPanelState()

        ' Reload current employee details
        LoadEmployeeDetails(Nothing)
    End Sub

End Class
